import _defineProperty from "@babel/runtime/helpers/defineProperty";
import _toConsumableArray from "@babel/runtime/helpers/toConsumableArray";
import _objectSpread from "@babel/runtime/helpers/objectSpread";

/**************************************************
 * Created by nanyuantingfeng on 11/06/2017 10:21.
 **************************************************/
import notifier from 'node-notifier';
import chalk from 'chalk';
import fs from 'fs';
import path from 'path';
export { notifier, chalk };
export function getProgressHandler(percent, msg1, msg2) {
  var stream = process.stdout;

  if (stream.isTTY && percent < 0.7) {
    stream.cursorTo(0);
    stream.write("\u231B  ".concat(chalk.magenta(msg2), " ").concat(msg1));
    stream.clearLine(1);
  } else if (percent >= 1) {
    console.log(chalk.green('\nwebpack: bundle build is now finished'));
  }
}
export function checkWebpackConfig(webpackConfig) {
  var configs = Array.isArray(webpackConfig) ? webpackConfig : [webpackConfig];
  var hasEmptyEntry = configs.some(function (c) {
    return Object.keys(c.entry || {}).length === 0;
  });

  if (hasEmptyEntry) {
    var e = new Error('no webpack entry found');
    e.name = 'NoEntry';
    throw e;
  }
}
export function getValueByPath(path) {
  return !fs.existsSync(path) ? {} : require(path);
}
export function getBuildCopyFiles(files) {
  if (Array.isArray(files)) {
    return files.map(function (from) {
      return {
        from: from
      };
    });
  }

  return Object.keys(files).filter(function (key) {
    var file = files[key];
    return typeof file === 'string' || !!file.path;
  }).map(function (key) {
    var file = files[key];

    if (typeof file === 'string') {
      return {
        from: file
      };
    }

    return {
      from: file.path,
      to: file.to
    };
  });
}
export function getBuildExternals(files) {
  var ret = {};
  Object.keys(files).forEach(function (key) {
    var file = files[key];

    if (typeof file === 'string') {
      ret[key] = file;
    } else if (file.name) {
      ret[key] = file.name;
    }
  });
  return ret;
}
export function getBuild4DevelopmentENV(filesMap) {
  var ret = [];
  Object.keys(filesMap).forEach(function (key) {
    var line = filesMap[key];
    var path = line.path;

    if (line.name && path) {
      ret.push(path);
    }
  });
  return ret;
}
export function getBuild4ProductionENV(filesMap) {
  var ret = [];
  Object.keys(filesMap).forEach(function (key) {
    var line = filesMap[key];
    var path = line.path;

    if (line.name && path) {
      var paths = path.split('/');
      path = paths[paths.length - 1];
      ret.push(path);
    }
  });
  return ret;
}
export function getBuildHTMLData(filesMap, env) {
  switch (env) {
    case 'production':
    case 'beta':
      return getBuild4ProductionENV(filesMap);

    default:
      return getBuild4DevelopmentENV(filesMap);
  }
}
export function getBuildHTML(context) {
  var _context$externals = context.externals,
      externals = _context$externals === void 0 ? {} : _context$externals,
      _context$sdks = context.sdks,
      sdks = _context$sdks === void 0 ? {} : _context$sdks,
      DLL_FILENAME = context.DLL_FILENAME,
      ENV = context.ENV,
      htmlWebpackPluginOptions = context.htmlWebpackPluginOptions,
      compress = context.compress;
  var entry = context.entry || context.packageMap.entry || getExampleEntry(context);

  if (!entry) {
    throw new Error('entry is an invalid value');
  }

  if (typeof entry === 'string') {
    context.entry = entry = {
      index: entry
    };
  }

  var paths = getBuildHTMLData(externals, ENV.env);
  var entryNames = Object.keys(entry);

  var options = _objectSpread({
    chunksSortMode: 'none',
    template: path.join(__dirname, '../assets/index.hbs'),
    favicon: path.join(__dirname, '../assets/favicon.ico')
  }, htmlWebpackPluginOptions);

  return entryNames.map(function (entryName) {
    var sdk = sdks[entryName];

    if (typeof sdk === 'string') {
      sdk = [sdk];
    }

    var scripts = paths.slice(0);

    if (sdk) {
      scripts.push.apply(scripts, _toConsumableArray(sdk));
    }

    if (DLL_FILENAME) {
      scripts.push(DLL_FILENAME);
    }

    return _objectSpread({
      PATHS: scripts,
      scripts: scripts,
      entryName: entryName,
      filename: "".concat(entryName, ".html"),
      chunks: [entryName],
      chunksSortMode: 'dependency',
      inject: true,
      templateParameters: getBuildTemplateParametersWithScripts(scripts),
      minify: compress ? {
        removeComments: true,
        collapseWhitespace: true,
        removeRedundantAttributes: true,
        useShortDoctype: true,
        removeEmptyAttributes: true,
        removeStyleLinkTypeAttributes: true,
        keepClosingSlash: true,
        minifyJS: true,
        minifyCSS: true,
        minifyURLs: true
      } : null
    }, options);
  });
}
export function createDomain(_ref) {
  var host = _ref.host,
      port = _ref.port;
  return "http://".concat(host, ":").concat(port);
}
export function getNodeVersion(packageMap) {
  var emptyBuildIns = ['child_process', 'cluster', 'dgram', 'dns', 'fs', 'module', 'net', 'readline', 'repl', 'tls'];
  var browser = packageMap.browser || {};
  return emptyBuildIns.reduce(function (obj, name) {
    if (!(name in browser)) {
      return _objectSpread({}, obj, _defineProperty({}, name, 'empty'));
    }

    return obj;
  }, {});
}
export function getBuildSourceMap() {
  var devtool = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
  var ENV = arguments.length > 1 ? arguments[1] : undefined;

  /******************
   *#source-map 编译过慢
   * production 环境不需要
   * beta 环境需要
   */
  if (devtool === true) {
    devtool = ENV.isProduction ? false : '#cheap-module-eval-source-map';
  }

  return devtool;
}
export function getBuildTemplateParametersWithScripts(scripts) {
  return function (compilation, assets, options) {
    var entryName = options.entryName;
    var stats = compilation.getStats().toJson();
    var currentAssets = stats.entrypoints[entryName].assets;
    var js = currentAssets.filter(function (n) {
      return path.extname(n) === '.js';
    });
    var css = currentAssets.filter(function (n) {
      return path.extname(n) === '.css';
    });
    assets.js = unique(scripts.concat(assets.js).concat(js));
    assets.css = unique(assets.css.concat(css));
    return {
      compilation: compilation,
      webpack: compilation.getStats().toJson(),
      webpackConfig: compilation.options,
      htmlWebpackPlugin: {
        files: assets,
        options: options,
        scripts: js,
        styles: css
      }
    };
  };
}

function unique(array) {
  return array.filter(function (item, index, array) {
    return array.indexOf(item) === index;
  });
}

export function getOptions(options) {
  var defaultOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return options === true ? defaultOptions : options;
}
export function PromiseDefer() {
  var resolve = void 0;
  var reject = void 0;
  var promise = new Promise(function (rs, rj) {
    resolve = rs;
    reject = rj;
  });
  return {
    promise: promise,
    resolve: resolve,
    reject: reject
  };
}
export function getExampleEntry(context) {
  if (!context.ENV.isDevelopment) {
    return null;
  }

  var entry = null;
  ['js', 'jsx', 'ts', 'tsx'].forEach(function (ext) {
    if (fs.existsSync(path.join(context.DIRs.root, 'example', "index.".concat(ext)))) {
      entry = "./example/index.".concat(ext);
    }
  });
  return entry;
}