"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = mwRules;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _path = _interopRequireDefault(require("path"));

var _plugins = require("./plugins");

var _miniCssExtractPlugin = _interopRequireDefault(require("mini-css-extract-plugin"));

/**************************************************
 * Created by nanyuantingfeng on 27/10/2017 16:28.
 **************************************************/
var REG_NODE_MODULES = /node_modules/;

function getThemeMap(packageMap, cwd) {
  var theme = {};
  var packageMapTheme = packageMap.theme;

  if (packageMapTheme && typeof packageMapTheme === 'string') {
    var pp = packageMapTheme;

    if (pp.charAt(0) === '.') {
      pp = _path["default"].resolve(cwd, pp);
    }

    var fn = require(pp);

    theme = fn();
  } else if (packageMapTheme && (0, _typeof2["default"])(packageMapTheme) === 'object') {
    theme = packageMapTheme;
  }

  return theme;
}

function mwRules(_x, _x2) {
  return _mwRules.apply(this, arguments);
}

function _mwRules() {
  _mwRules = (0, _asyncToGenerator2["default"])(
  /*#__PURE__*/
  _regenerator["default"].mark(function _callee(context, next) {
    var cwd, _context$limit, limit, ENV, packageMap, hash, plugins, babelOptions, theme, postcssOptions, rules, scriptRules, stylesRules, othersRules, filename, chunkFilename;

    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            context.rules = [];
            next();
            cwd = context.cwd, _context$limit = context.limit, limit = _context$limit === void 0 ? 10240 : _context$limit, ENV = context.ENV, packageMap = context.packageMap, hash = context.hash, plugins = context.plugins, babelOptions = context.babelOptions;
            theme = getThemeMap(packageMap, cwd);
            postcssOptions = context.postcssOptions, rules = context.rules;
            scriptRules = [{
              test: /\.worker\.[tj]sx?$/,
              exclude: [REG_NODE_MODULES],
              use: [{
                loader: 'workerize-loader'
              }, {
                loader: 'babel-loader',
                options: babelOptions
              }]
            }, {
              test: function test(filePath) {
                return /\.[tj]sx?$/.test(filePath) && !/\.worker\.[tj]sx?$/.test(filePath);
              },
              exclude: [REG_NODE_MODULES, /@babel(?:\/|\\{1,2})runtime/],
              use: [{
                loader: 'babel-loader',
                options: babelOptions
                /*, getReplaceLodashLoader()*/

              }]
            }, {
              test: /\.mjsx?$/,
              include: [REG_NODE_MODULES],
              type: 'javascript/auto'
            }, {
              test: /\.(graphql|gql)$/,
              exclude: [REG_NODE_MODULES],
              use: [{
                loader: 'graphql-tag/loader'
              }]
            }];
            stylesRules = [{
              test: function test(filePath) {
                return /\.css$/.test(filePath) && !/\.module\.css$/.test(filePath);
              },
              use: [{
                loader: 'style-loader'
              }, {
                loader: 'css-loader',
                options: {
                  sourceMap: true
                }
              }, {
                loader: 'postcss-loader',
                options: postcssOptions
              }]
            }, {
              test: /\.module\.css$/,
              use: [{
                loader: 'style-loader'
              }, {
                loader: 'css-loader',
                options: {
                  sourceMap: true,
                  modules: true,
                  localIdentName: '[local]___[hash:base64:5]'
                }
              }, {
                loader: 'postcss-loader',
                options: postcssOptions
              }]
            }, {
              test: function test(filePath) {
                return /\.less$/.test(filePath) && !/\.module\.less$/.test(filePath);
              },
              use: [{
                loader: 'style-loader'
              }, {
                loader: 'css-loader',
                options: {
                  sourceMap: true
                }
              }, {
                loader: 'postcss-loader',
                options: postcssOptions
              }, {
                loader: 'less-loader',
                options: {
                  javascriptEnabled: true,
                  sourceMap: true,
                  modifyVars: theme
                }
              }]
            }, {
              test: /\.module\.less$/,
              use: [{
                loader: 'style-loader'
              }, {
                loader: 'css-loader',
                options: {
                  sourceMap: true,
                  modules: true,
                  localIdentName: '[local]___[hash:base64:5]'
                }
              }, {
                loader: 'postcss-loader',
                options: postcssOptions
              }, {
                loader: 'less-loader',
                options: {
                  javascriptEnabled: true,
                  sourceMap: true,
                  modifyVars: theme
                }
              }]
            }];
            othersRules = [{
              test: /\.svgx$/,
              use: [{
                loader: '@svgr/webpack'
              }]
            }, {
              test: /\.json5$/,
              use: [{
                loader: 'json5-loader'
              }]
            }, {
              test: /\.(woff|woff2)?(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'application/font-woff'
                }
              }]
            }, {
              test: /\.ttf(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'application/octet-stream'
                }
              }]
            }, {
              test: /\.eot(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'application/vnd.ms-fontobject'
                }
              }]
            }, {
              test: /\.svg(\?v=\d+\.\d+\.\d+)?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  limit: limit,
                  mimetype: 'image/svg+xml'
                }
              }]
            }, {
              test: /\.(bmp|png|jpe?g|gif)(\?v=\d+\.\d+\.\d+)?$/i,
              use: [{
                loader: 'url-loader',
                options: {
                  limit: limit,
                  name: 'static/media/[name].[hash:8].[ext]'
                }
              }]
            }, {
              test: /\.html?$/,
              use: [{
                loader: 'file-loader',
                options: {
                  name: '[path]-[name].[ext]'
                }
              }]
            }, {
              test: /\.hbs?$/,
              use: [{
                loader: 'mustache-loader'
              }]
            }];

            if (ENV.isProduction || ENV.isBeta) {
              stylesRules.map(function (rule) {
                return rule.use[0] = _miniCssExtractPlugin["default"].loader;
              });
              filename = hash ? '[name]-[contenthash:8].css' : '[name].css';
              chunkFilename = hash ? '[name]-[contenthash:8].chunk.css' : '[name].chunk.css';
              plugins.push(new _miniCssExtractPlugin["default"]({
                filename: filename,
                chunkFilename: chunkFilename
              }));
            }

            context.rules = scriptRules // @ts-ignore
            .concat([{
              parser: {
                requireEnsure: false
              }
            }]).concat(stylesRules).concat(othersRules).concat(rules);
            plugins.push(new _plugins.WatchIgnorePlugin([/\.d\.ts$/]));

          case 11:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _mwRules.apply(this, arguments);
}